import { NativeModulesProxy, EventEmitter, Subscription } from 'expo-modules-core';

// Import the native module. On web, it will be resolved to LibPoc.web.ts
// and on native platforms to LibPoc.ts
import LibPoc from './LibPocModule';
import LibPocView from './LibPocView';
import { ChangeEventPayload, LibPocViewProps } from './LibPoc.types';

// Get the native constant value.
export const PI = LibPoc.PI;

export function hello(): string {
  return LibPoc.hello();
}

export async function setValueAsync(value: string) {
  return await LibPoc.setValueAsync(value);
}

// For now the events are not going through the JSI, so we have to use its bridge equivalent.
// This will be fixed in the stable release and built into the module object.
// Note: On web, NativeModulesProxy.LibPoc is undefined, so we fall back to the directly imported implementation
const emitter = new EventEmitter(NativeModulesProxy.LibPoc ?? LibPoc);

export function addChangeListener(listener: (event: ChangeEventPayload) => void): Subscription {
  return emitter.addListener<ChangeEventPayload>('onChange', listener);
}

export { LibPocView, LibPocViewProps, ChangeEventPayload };
